@@ -49,11 +49,12 @@ def serialize(x: Any) -> Any:
 
 
 def serialize_test_case(test_case: BaseTestCase) -> Any:
+    obj_to_serialize = test_case.serialize()
     # See https://docs.python.org/3/library/dataclasses.html#dataclasses.is_dataclass:
     # isinstance(test_case, type) checks test_case is an instance and not a type
-    if dataclasses.is_dataclass(test_case) and not isinstance(test_case, type):
+    if dataclasses.is_dataclass(obj_to_serialize) and not isinstance(obj_to_serialize, type):
         serialized: dict[Any, Any] = {}
-        for k, v in dataclasses.asdict(test_case).items():
+        for k, v in dataclasses.asdict(obj_to_serialize).items():
             if k == TEST_CASE_CONFIG_ATTR:
                 # Don't serialize the config
                 continue
@@ -64,7 +65,13 @@ def serialize_test_case(test_case: BaseTestCase) -> Any:
                 pass
         return serialized
 
-    return serialize(test_case)
+    return serialize(obj_to_serialize)
+
+
+def serialize_output(output: Any) -> Any:
+    if callable(getattr(output, "serialize", None)):
+        return serialize(output.serialize())
+    return serialize(output)
 
 
 def serialize_human_review_fields(fields: Optional[list[HumanReviewField]]) -> Optional[list[dict[str, str]]]:
