@@ -21,16 +21,12 @@ from autoblocks._impl.testing.models import Evaluation
 from autoblocks._impl.testing.models import HumanReviewField
 from autoblocks._impl.testing.models import TracerEvent
 from autoblocks._impl.testing.util import serialize_human_review_fields
-from autoblocks._impl.util import AnyTask
 from autoblocks._impl.util import AutoblocksEnvVar
 from autoblocks._impl.util import all_settled
-from autoblocks._impl.util import get_running_loop
 from autoblocks._impl.util import now_iso_8601
 
 log = logging.getLogger(__name__)
 
-evaluator_semaphore_registry: dict[str, asyncio.Semaphore] = {}  # evaluator_id -> semaphore
-
 
 @dataclasses.dataclass(frozen=True)
 class Payload:
@@ -50,6 +46,10 @@ class Payload:
         }
 
 
+evaluator_semaphore_registry: dict[str, asyncio.Semaphore] = {}  # evaluator_id -> semaphore
+test_events: dict[Tuple[str, str], List[Payload]] = {}  # run_id + test_case_hash -> list of events
+
+
 class AutoblocksTracer:
     def __init__(
         self,
@@ -222,6 +222,8 @@ class AutoblocksTracer:
             if global_state.main_thread_has_finished():
                 # If we're in a shutdown state, we need to use the sync client
                 # to avoid scheduling new futures after the interpreter has shut down.
+                # There is a race condition that can happen where the main thread finishes right as we check
+                # This happens in our async_script e2e test, so we have to use flush()
                 # See https://github.com/boto/boto3/issues/3113
                 log.debug("Sending event with SYNC client because interpreter has shut down")
                 req = global_state.sync_http_client().post(
@@ -252,25 +254,15 @@ class AutoblocksTracer:
         payload: Payload,
     ) -> None:
         """Sends an event to the Autoblocks CLI in a testing environment."""
-        if not self._cli_server_address:
-            log.error("Failed to send test event to Autoblocks. CLI server address not set.")
-            return
         test_case_run = test_case_run_context_var.get()
         if not test_case_run:
             log.error("Failed to send test event to Autoblocks. No test ID set.")
             return
-
-        req = global_state.sync_http_client().post(
-            url=f"{self._cli_server_address}/events",
-            json=dict(
-                testExternalId=test_case_run.test_id,
-                runId=test_case_run.run_id,
-                testCaseHash=test_case_run.test_case_hash,
-                event=payload.to_json(),
-            ),
-            timeout=self._timeout_seconds,
-        )
-        req.raise_for_status()
+        # Initialize the list for this run id and test case hash if it doesn't exist
+        run_id_and_test_case_hash = (test_case_run.run_id, test_case_run.test_case_hash)
+        if run_id_and_test_case_hash not in test_events:
+            test_events[run_id_and_test_case_hash] = []
+        test_events[run_id_and_test_case_hash].append(payload)
 
     def _make_request_payload(
         self,
@@ -315,26 +307,14 @@ class AutoblocksTracer:
         payload: Payload,
         evaluators: Optional[Sequence[BaseEventEvaluator]],
     ) -> None:
-        task: AnyTask
-
-        # Check if we're already in a running event loop
-        if running_loop := get_running_loop():
-            # If we are, execute the task on that loop
-            task = running_loop.create_task(
-                self._async_send_event(
-                    payload=payload,
-                    evaluators=evaluators,
-                ),
-            )
-        else:
-            # Otherwise, send the task to our background loop
-            task = asyncio.run_coroutine_threadsafe(
-                self._async_send_event(
-                    payload=payload,
-                    evaluators=evaluators,
-                ),
-                global_state.event_loop(),
-            )
+        # Send the task to our background loop
+        task = asyncio.run_coroutine_threadsafe(
+            self._async_send_event(
+                payload=payload,
+                evaluators=evaluators,
+            ),
+            global_state.event_loop(),
+        )
 
         # Add the task to the set of tasks the
         # global state will wait to complete
