@@ -30,6 +30,7 @@ class AutoblocksEnvVar(StrEnum):
     API_KEY = "AUTOBLOCKS_API_KEY"
     CLI_SERVER_ADDRESS = "AUTOBLOCKS_CLI_SERVER_ADDRESS"
     INGESTION_KEY = "AUTOBLOCKS_INGESTION_KEY"
+    FILTERS_TEST_SUITES = "AUTOBLOCKS_FILTERS_TEST_SUITES"
     OVERRIDES_PROMPT_REVISIONS = "AUTOBLOCKS_OVERRIDES_PROMPT_REVISIONS"
     OVERRIDES_CONFIG_REVISIONS = "AUTOBLOCKS_OVERRIDES_CONFIG_REVISIONS"
     OVERRIDES_TESTS_AND_HASHES = "AUTOBLOCKS_OVERRIDES_TESTS_AND_HASHES"
@@ -46,6 +47,10 @@ class ThirdPartyEnvVar(StrEnum):
         return os.environ.get(self.value)
 
 
+def is_ci() -> bool:
+    return os.environ.get("CI") == "true"
+
+
 def encode_uri_component(s: str) -> str:
     """
     This should have the same behavior as encodeURIComponent from JS.
@@ -78,3 +83,7 @@ def get_running_loop() -> Optional[asyncio.AbstractEventLoop]:
 
 def now_iso_8601() -> str:
     return datetime.now(timezone.utc).isoformat()
+
+
+def is_cli_running() -> bool:
+    return AutoblocksEnvVar.CLI_SERVER_ADDRESS.get() is not None
