@@ -1,3 +1,4 @@
+from autoblocks._impl.testing.evaluators.accuracy import BaseAccuracy
 from autoblocks._impl.testing.evaluators.battle import BaseAutomaticBattle
 from autoblocks._impl.testing.evaluators.battle import BaseAutomaticBattle as AutomaticBattle
 from autoblocks._impl.testing.evaluators.battle import BaseManualBattle
@@ -14,7 +15,6 @@ from autoblocks._impl.testing.evaluators.ragas_answer_semantic_similarity import
 from autoblocks._impl.testing.evaluators.ragas_context_entities_recall import BaseRagasContextEntitiesRecall
 from autoblocks._impl.testing.evaluators.ragas_context_precision import BaseRagasContextPrecision
 from autoblocks._impl.testing.evaluators.ragas_context_recall import BaseRagasContextRecall
-from autoblocks._impl.testing.evaluators.ragas_context_relevancy import BaseRagasContextRelevancy
 from autoblocks._impl.testing.evaluators.ragas_faithfulness import BaseRagasFaithfulness
 from autoblocks._impl.testing.evaluators.toxicity import BaseToxicity
 
@@ -30,9 +30,9 @@ __all__ = [
     "BaseRagasContextEntitiesRecall",
     "BaseRagasContextPrecision",
     "BaseRagasContextRecall",
-    "BaseRagasContextRelevancy",
     "BaseRagasFaithfulness",
     "BaseLLMJudge",
+    "BaseAccuracy",
     "BaseNSFW",
     "BaseToxicity",
     # deprecated
